@@ -39,7 +39,7 @@ def depends(*args, **kwargs):
     Python decorator annotating a function or `Parameterized` method to
     express its dependencies on a set of Parameters.
 
-    Despite still being available usage of `pn.depends` is no longer
+    Despite still being available, usage of `pn.depends` is no longer
     recommended, in favor of the less intrusive `pn.bind`.
 
     Returns a "reactive" function that binds (some of) its arguments to
@@ -47,8 +47,6 @@ def depends(*args, **kwargs):
     (or will if `watch=True`) be automatically invoked whenever the underlying
     parameter values change.
 
-    Reference: https://panel.holoviz.org/user_guide/APIs.html#reactive-functions
-
     :Example:
 
     >>> widget = pn.widgets.IntSlider(value=1, start=1, end=5)
@@ -88,7 +86,7 @@ def bind(function, *args, watch=False, **kwargs):
     (or will if `watch=True`) be automatically invoked whenever the underlying
     parameter values change.
 
-    Reference: https://panel.holoviz.org/user_guide/APIs.html#reactive-functions
+    How-to: https://panel.holoviz.org/how_to/interactivity/bind_function.html
 
     :Example:
 
