@@ -4,6 +4,7 @@ import asyncio
 import dataclasses
 import datetime as dt
 import inspect
+import json
 import logging
 import threading
 
@@ -16,7 +17,10 @@ from typing import (
 from bokeh.application.application import SessionContext
 from bokeh.document.document import Document
 from bokeh.document.events import DocumentChangedEvent, ModelChangedEvent
+from bokeh.models import CustomJS
 
+from ..config import config
+from .loading import LOADING_INDICATOR_CSS_CLASS
 from .model import monkeypatch_events
 from .state import curdoc_locked, state
 
@@ -59,7 +63,7 @@ def _dispatch_events(doc: Document, events: List[DocumentChangedEvent]) -> None:
     for event in events:
         doc.callbacks.trigger_on_change(event)
 
-def _cleanup_doc(doc):
+def _cleanup_doc(doc, destroy=True):
     for callback in doc.session_destroyed_callbacks:
         try:
             callback(None)
@@ -80,31 +84,42 @@ def _cleanup_doc(doc):
                     p._hooks = []
                     p._param_watchers = {}
                     p._documents = {}
-                    p._callbacks = {}
+                    p._internal_callbacks = {}
             pane._param_watchers = {}
             pane._documents = {}
-            pane._callbacks = {}
+            pane._internal_callbacks = {}
         else:
             views[ref] = (pane, root, doc, comm)
     state._views = views
 
+    # When reusing sessions we must clean up the Panel state but we
+    # must **not** destroy the template or the document
+    if not destroy:
+        return
+
     # Clean up templates
     if doc in state._templates:
         tmpl = state._templates[doc]
         tmpl._documents = {}
         del state._templates[doc]
 
-    # Destroy doc
+    # Destroy document
     doc.destroy(None)
 
 #---------------------------------------------------------------------
 # Public API
 #---------------------------------------------------------------------
 
-def init_doc(doc: Optional[Document]) -> Document:
+def create_doc_if_none_exists(doc: Optional[Document]) -> Document:
     curdoc = doc or curdoc_locked()
-    if not isinstance(curdoc, Document):
+    if curdoc is None:
+        curdoc = Document()
+    elif not isinstance(curdoc, Document):
         curdoc = curdoc._doc
+    return curdoc
+
+def init_doc(doc: Optional[Document]) -> Document:
+    curdoc = create_doc_if_none_exists(doc)
     if not curdoc.session_context:
         return curdoc
 
@@ -112,6 +127,14 @@ def init_doc(doc: Optional[Document]) -> Document:
     if thread:
         state._thread_id_[curdoc] = thread.ident
 
+    if config.global_loading_spinner:
+        curdoc.js_on_event(
+            'document_ready', CustomJS(code=f"""
+            const body = document.getElementsByTagName('body')[0]
+            body.classList.remove({LOADING_INDICATOR_CSS_CLASS!r}, {config.loading_spinner!r})
+            """)
+        )
+
     session_id = curdoc.session_context.id
     sessions = state.session_info['sessions']
     if session_id not in sessions:
@@ -149,34 +172,38 @@ def with_lock(func: Callable) -> Callable:
     wrapper.lock = True # type: ignore
     return wrapper
 
-def dispatch_tornado(conn, event):
+def dispatch_tornado(conn, events):
     from tornado.websocket import WebSocketHandler
     socket = conn._socket
-    ws_conn = socket.ws_connection
+    ws_conn = getattr(socket, 'ws_connection', False)
     if not ws_conn or ws_conn.is_closing(): # type: ignore
         return []
-    msg = conn.protocol.create('PATCH-DOC', [event])
+    msg = conn.protocol.create('PATCH-DOC', events)
     futures = [
         WebSocketHandler.write_message(socket, msg.header_json),
         WebSocketHandler.write_message(socket, msg.metadata_json),
         WebSocketHandler.write_message(socket, msg.content_json)
     ]
-    for header, payload in msg._buffers:
+    for buffer in msg._buffers:
+        header = json.dumps(buffer.ref)
+        payload = buffer.to_bytes()
         futures.extend([
             WebSocketHandler.write_message(socket, header),
             WebSocketHandler.write_message(socket, payload, binary=True)
         ])
     return futures
 
-def dispatch_django(conn, event):
+def dispatch_django(conn, events):
     socket = conn._socket
-    msg = conn.protocol.create('PATCH-DOC', [event])
+    msg = conn.protocol.create('PATCH-DOC', events)
     futures = [
         socket.send(text_data=msg.header_json),
         socket.send(text_data=msg.metadata_json),
         socket.send(text_data=msg.content_json)
     ]
-    for header, payload in msg._buffers:
+    for buffer in msg._buffers:
+        header = json.dumps(buffer.ref)
+        payload = buffer.to_bytes()
         futures.extend([
             socket.send(text_data=header),
             socket.send(binary_data=payload)
@@ -217,16 +244,21 @@ def unlocked() -> Iterator:
 
         events = curdoc.callbacks._held_events
         monkeypatch_events(events)
-        remaining_events, futures = [], []
+        remaining_events, dispatch_events = [], []
         for event in events:
-            if not isinstance(event, ModelChangedEvent) or locked:
+            if isinstance(event, ModelChangedEvent) and not locked:
+                dispatch_events.append(event)
+            else:
                 remaining_events.append(event)
+
+        futures = []
+        for conn in connections:
+            if not dispatch_events:
                 continue
-            for conn in connections:
-                if isinstance(conn._socket, WebSocketHandler):
-                    futures += dispatch_tornado(conn, event)
-                else:
-                    futures += dispatch_django(conn, event)
+            elif isinstance(conn._socket, WebSocketHandler):
+                futures += dispatch_tornado(conn, dispatch_events)
+            else:
+                futures += dispatch_django(conn, dispatch_events)
 
         # Ensure that all write_message calls are awaited and handled
         async def handle_write_errors():
@@ -238,17 +270,19 @@ def unlocked() -> Iterator:
                 except Exception as e:
                     logger.warning(f"Failed sending message due to following error: {e}")
 
-        if state._unblocked(curdoc):
-            try:
-                asyncio.ensure_future(handle_write_errors())
-            except RuntimeError:
+        if futures:
+            if state._unblocked(curdoc):
+                try:
+                    asyncio.ensure_future(handle_write_errors())
+                except RuntimeError:
+                    curdoc.add_next_tick_callback(handle_write_errors)
+            else:
                 curdoc.add_next_tick_callback(handle_write_errors)
-        else:
-            curdoc.add_next_tick_callback(handle_write_errors)
 
         curdoc.callbacks._held_events = remaining_events
     finally:
         try:
             curdoc.unhold()
         except RuntimeError:
-            curdoc.add_next_tick_callback(partial(_dispatch_events, curdoc, remaining_events))
+            if remaining_events:
+                curdoc.add_next_tick_callback(partial(_dispatch_events, curdoc, remaining_events))
