@@ -1,15 +1,13 @@
-import * as p from "@bokehjs/core/properties"
-
 import {div} from "@bokehjs/core/dom"
+import * as p from "@bokehjs/core/properties"
 import {clone} from "@bokehjs/core/util/object"
-import {isEqual} from "@bokehjs/core/util/eq"
-import {HTMLBox} from "@bokehjs/models/layouts/html_box"
+import {is_equal} from "@bokehjs/core/util/eq"
 import {ColumnDataSource} from "@bokehjs/models/sources/column_data_source";
 
 import {debounce} from  "debounce"
 import {deepCopy, isPlainObject, get, throttle} from "./util"
 
-import {PanelHTMLBoxView, set_size} from "./layout"
+import {HTMLBox, HTMLBoxView, set_size} from "./layout"
 
 
 interface PlotlyHTMLElement extends HTMLDivElement {
@@ -40,7 +38,7 @@ const filterEventData = (gd: any, eventData: any, event: string) => {
         /*
          * remove `data`, `layout`, `xaxis`, etc
          * objects from the event data since they're so big
-         * and cause JSON stringify ciricular structure errors.
+         * and cause JSON stringify circular structure errors.
          *
          * also, pull down the `customdata` point from the data array
          * into the event object
@@ -109,24 +107,20 @@ const _isHidden = (gd: any) => {
 };
 
 
-export class PlotlyPlotView extends PanelHTMLBoxView {
+export class PlotlyPlotView extends HTMLBoxView {
   model: PlotlyPlot
   _setViewport: Function
   _settingViewport: boolean = false
   _plotInitialized: boolean = false
+  _rendered: boolean = false
   _reacting: boolean = false
   _relayouting: boolean = false
-  _layout_wrapper: PlotlyHTMLElement
+  container: PlotlyHTMLElement
   _watched_sources: string[]
   _end_relayouting = debounce(() => {
     this._relayouting = false
   }, 2000, false)
 
-  initialize(): void {
-    super.initialize()
-    this._layout_wrapper = <PlotlyHTMLElement>div({style: "height: 100%; width: 100%;"})
-  }
-
   connect_signals(): void {
     super.connect_signals();
     const {data, data_sources, layout, relayout, restyle} = this.model.properties
@@ -140,13 +134,13 @@ export class PlotlyPlotView extends PanelHTMLBoxView {
     this.on_change([relayout], () => {
       if (this.model.relayout == null)
         return
-      (window as any).Plotly.relayout(this._layout_wrapper, this.model.relayout)
+      (window as any).Plotly.relayout(this.container, this.model.relayout)
       this.model.relayout = null
     })
     this.on_change([restyle], () => {
       if (this.model.restyle == null)
         return
-      (window as any).Plotly.restyle(this._layout_wrapper, this.model.restyle.data, this.model.restyle.traces)
+      (window as any).Plotly.restyle(this.container, this.model.restyle.data, this.model.restyle.traces)
       this.model.restyle = null
     })
 
@@ -163,17 +157,37 @@ export class PlotlyPlotView extends PanelHTMLBoxView {
       this.plot(true)
     });
     this.connect(this.model.properties.viewport.change, () => this._updateViewportFromProperty());
-    this.connect(this.model.properties.visibility.change, () => {
-      this.el.style.visibility = this.model.visibility ? 'visible' : 'hidden'
+  }
+
+  override remove(): void {
+    (window as any).Plotly.purge(this.container)
+    super.remove()
+  }
+
+  render(): void {
+    super.render()
+    this.container = <PlotlyHTMLElement>div()
+    set_size(this.container, this.model)
+    this._rendered = false
+    this.shadow_el.appendChild(this.container)
+    this.watch_stylesheets()
+    this.plot().then(() => {
+      this._rendered = true
+      if (this.model.relayout != null)
+	(window as any).Plotly.relayout(this.container, this.model.relayout);
+      (window as any).Plotly.Plots.resize(this.container);
     })
   }
 
-  async render(): Promise<void> {
-    super.render()
-    this.el.style.visibility = this.model.visibility ? 'visible' : 'hidden'
-    this.el.appendChild(this._layout_wrapper)
-    await this.plot();
-    (window as any).Plotly.relayout(this._layout_wrapper, this.model.relayout)
+  style_redraw(): void {
+    if (this._rendered)
+      (window as any).Plotly.Plots.resize(this.container);
+  }
+
+  after_layout(): void {
+    super.after_layout();
+    if (this._rendered)
+      (window as any).Plotly.Plots.resize(this.container)
   }
 
   _trace_data(): any {
@@ -186,7 +200,7 @@ export class PlotlyPlotView extends PanelHTMLBoxView {
   _layout_data(): any {
     const newLayout = deepCopy(this.model.layout);
     if (this._relayouting) {
-      const {layout} = this._layout_wrapper;
+      const {layout} = this.container;
 
       // For each xaxis* and yaxis* property of layout, if the value has a 'range'
       // property then use this in newLayout
@@ -201,10 +215,10 @@ export class PlotlyPlotView extends PanelHTMLBoxView {
 
   _install_callbacks(): void {
     //  - plotly_relayout
-    this._layout_wrapper.on('plotly_relayout', (eventData: any) => {
+    this.container.on('plotly_relayout', (eventData: any) => {
       if (eventData['_update_from_property'] !== true) {
         this.model.relayout_data = filterEventData(
-          this._layout_wrapper, eventData, 'relayout');
+          this.container, eventData, 'relayout');
 
         this._updateViewportProperty();
 
@@ -213,7 +227,7 @@ export class PlotlyPlotView extends PanelHTMLBoxView {
     });
 
     //  - plotly_relayouting
-    this._layout_wrapper.on('plotly_relayouting', () => {
+    this.container.on('plotly_relayouting', () => {
       if (this.model.viewport_update_policy !== 'mouseup') {
         this._relayouting = true;
         this._updateViewportProperty();
@@ -221,45 +235,45 @@ export class PlotlyPlotView extends PanelHTMLBoxView {
     });
 
     //  - plotly_restyle
-    this._layout_wrapper.on('plotly_restyle', (eventData: any) => {
+    this.container.on('plotly_restyle', (eventData: any) => {
       this.model.restyle_data = filterEventData(
-        this._layout_wrapper, eventData, 'restyle');
+        this.container, eventData, 'restyle');
 
       this._updateViewportProperty();
     });
 
     //  - plotly_click
-    this._layout_wrapper.on('plotly_click', (eventData: any) => {
+    this.container.on('plotly_click', (eventData: any) => {
       this.model.click_data = filterEventData(
-        this._layout_wrapper, eventData, 'click');
+        this.container, eventData, 'click');
     });
 
     //  - plotly_hover
-    this._layout_wrapper.on('plotly_hover', (eventData: any) => {
+    this.container.on('plotly_hover', (eventData: any) => {
       this.model.hover_data = filterEventData(
-        this._layout_wrapper, eventData, 'hover');
+        this.container, eventData, 'hover');
     });
 
     //  - plotly_selected
-    this._layout_wrapper.on('plotly_selected', (eventData: any) => {
+    this.container.on('plotly_selected', (eventData: any) => {
       this.model.selected_data = filterEventData(
-        this._layout_wrapper, eventData, 'selected');
+        this.container, eventData, 'selected');
     });
 
     //  - plotly_clickannotation
-    this._layout_wrapper.on('plotly_clickannotation', (eventData: any) => {
+    this.container.on('plotly_clickannotation', (eventData: any) => {
       delete eventData["event"];
       delete eventData["fullAnnotation"];
       this.model.clickannotation_data = eventData
     });
 
     //  - plotly_deselect
-    this._layout_wrapper.on('plotly_deselect', () => {
+    this.container.on('plotly_deselect', () => {
       this.model.selected_data = null;
     });
 
     //  - plotly_unhover
-    this._layout_wrapper.on('plotly_unhover', () => {
+    this.container.on('plotly_unhover', () => {
       this.model.hover_data = null;
     });
   }
@@ -272,29 +286,22 @@ export class PlotlyPlotView extends PanelHTMLBoxView {
     this._reacting = true
     if (new_plot) {
       const obj = {data: data, layout: newLayout, config: this.model.config, frames: this.model.frames}
-      await (window as any).Plotly.newPlot(this._layout_wrapper, obj)
+      await (window as any).Plotly.newPlot(this.container, obj)
     } else {
-      await (window as any).Plotly.react(this._layout_wrapper, data, newLayout, this.model.config)
+      await (window as any).Plotly.react(this.container, data, newLayout, this.model.config)
       if (this.model.frames != null)
-        await (window as any).Plotly.addFrames(this._layout_wrapper, this.model.frames)
+        await (window as any).Plotly.addFrames(this.container, this.model.frames)
     }
     this._updateSetViewportFunction()
     this._updateViewportProperty()
     if (!this._plotInitialized)
       this._install_callbacks()
-    else if (!_isHidden(this._layout_wrapper))
-      (window as any).Plotly.Plots.resize(this._layout_wrapper)
+    else if (!_isHidden(this.container))
+      (window as any).Plotly.Plots.resize(this.container)
     this._reacting = false
     this._plotInitialized = true
   }
 
-  after_layout(): void{
-    super.after_layout()
-    set_size(this.el, this.model)
-    if ((window as any).Plotly && this._plotInitialized)
-      (window as any).Plotly.Plots.resize(this._layout_wrapper)
-  }
-
   _get_trace(index: number, update: boolean): any {
     const trace = clone(this.model.data[index]);
     const cds = this.model.data_sources[index];
@@ -327,11 +334,11 @@ export class PlotlyPlotView extends PanelHTMLBoxView {
   _updateViewportFromProperty(): void {
     if (!(window as any).Plotly || this._settingViewport || this._reacting || !this.model.viewport ) { return }
 
-    const fullLayout = this._layout_wrapper._fullLayout;
+    const fullLayout = this.container._fullLayout;
 
     // Call relayout if viewport differs from fullLayout
     Object.keys(this.model.viewport).reduce((value: any, key: string) => {
-      if (!isEqual(get(fullLayout, key), value)) {
+      if (!is_equal(get(fullLayout, key), value)) {
         let clonedViewport = deepCopy(this.model.viewport)
         clonedViewport['_update_from_property'] = true
         this._settingViewport = true;
@@ -346,7 +353,7 @@ export class PlotlyPlotView extends PanelHTMLBoxView {
   }
 
   _updateViewportProperty(): void {
-    const fullLayout = this._layout_wrapper._fullLayout;
+    const fullLayout = this.container._fullLayout;
     let viewport: any = {};
 
     // Get range for all xaxis and yaxis properties
@@ -358,7 +365,7 @@ export class PlotlyPlotView extends PanelHTMLBoxView {
         viewport[prop + '.range'] = deepCopy(fullLayout[prop].range)
     }
 
-    if (!isEqual(viewport, this.model.viewport))
+    if (!is_equal(viewport, this.model.viewport))
       this._setViewport(viewport)
   }
 
@@ -403,7 +410,6 @@ export namespace PlotlyPlot {
     viewport: p.Property<any>
     viewport_update_policy: p.Property<string>
     viewport_update_throttle: p.Property<number>
-    visibility: p.Property<boolean>
     _render_count: p.Property<number>
   }
 }
@@ -419,10 +425,10 @@ export class PlotlyPlot extends HTMLBox {
 
   static __module__ = "panel.models.plotly"
 
-  static init_PlotlyPlot(): void {
+  static {
     this.prototype.default_view = PlotlyPlotView
 
-    this.define<PlotlyPlot.Props>(({Array, Any, Boolean, Ref, String, Nullable, Number}) => ({
+    this.define<PlotlyPlot.Props>(({Array, Any, Nullable, Number, Ref, String}) => ({
       data: [ Array(Any), [] ],
       layout: [ Any, {} ],
       config: [ Any, {} ],
@@ -439,7 +445,6 @@ export class PlotlyPlot extends HTMLBox {
       viewport: [ Any, {} ],
       viewport_update_policy: [ String, "mouseup" ],
       viewport_update_throttle: [ Number, 200 ],
-      visibility: [ Boolean, true],
       _render_count: [ Number, 0 ],
     }))
   }
