@@ -8,7 +8,7 @@ import os
 from base64 import b64encode
 from io import BytesIO
 from typing import (
-    TYPE_CHECKING, Any, ClassVar, List, Mapping, Optional,
+    Any, ClassVar, Dict, List, Mapping,
 )
 
 import numpy as np
@@ -16,18 +16,32 @@ import param
 
 from ..models import Audio as _BkAudio, Video as _BkVideo
 from ..util import isfile, isurl
-from .base import PaneBase
-
-if TYPE_CHECKING:
-    from bokeh.document import Document
-    from bokeh.model import Model
-    from pyviz_comms import Comm
+from .base import ModelPane
 
 
-class _MediaBase(PaneBase):
+class TensorLikeMeta(type):
+    """See https://blog.finxter.com/python-__instancecheck__-magic-method/"""
+    def __instancecheck__(self, instance):
+        numpy_attr = getattr(instance, "numpy", "")
+        dim_attr = getattr(instance, "dim", "")
+        return (
+            bool(numpy_attr) and
+            callable(numpy_attr) and
+            callable(dim_attr) and
+            hasattr(instance, "dtype")
+        )
+
+# When support for Python 3.7 is dropped, Torchlike should be implemented as a typing.Protocol
+# That would provide type checking and intellisense in editors
+class TensorLike(metaclass=TensorLikeMeta):
+    """A class similar to torch.Tensor. We don't want to make PyTorch a dependency of this project
+    """
+    pass
+
+class _MediaBase(ModelPane):
 
     loop = param.Boolean(default=False, doc="""
-        Whether the meida should loop""")
+        Whether the media should loop""")
 
     time = param.Number(default=0, doc="""
         The current timestamp""")
@@ -58,7 +72,8 @@ class _MediaBase(PaneBase):
     _media_type: ClassVar[str]
 
     _rename: ClassVar[Mapping[str, str | None]] = {
-        'name': None, 'sample_rate': None, 'object': 'value'}
+        'sample_rate': None, 'object': 'value'
+    }
 
     _rerender_params: ClassVar[List[str]] = []
 
@@ -77,19 +92,19 @@ class _MediaBase(PaneBase):
             return True
         return False
 
-    def _get_model(
-        self, doc: Document, root: Optional[Model] = None,
-        parent: Optional[Model] = None, comm: Optional[Comm] = None
-    ) -> Model:
-        props = self._process_param_change(self._init_params())
-        model = self._bokeh_model(**props)
-        if root is None:
-            root = model
-        self._models[root.ref['id']] = (model, parent)
-        self._link_props(model, list(model.properties()), doc, root, comm)
-        return model
+    def _to_np_int16(self, data: np.ndarray):
+        dtype = data.dtype
+
+        if dtype in (np.float32, np.float64):
+            data = (data * 32768.0).astype(np.int16)
+
+        return data
+
+    def _to_buffer(self, data: np.ndarray|TensorLike):
+        if isinstance(data, TensorLike):
+            data = data.numpy()
+        data = self._to_np_int16(data)
 
-    def _from_numpy(self, data):
         from scipy.io import wavfile
         buffer = BytesIO()
         wavfile.write(buffer, self.sample_rate, data)
@@ -101,40 +116,70 @@ class _MediaBase(PaneBase):
             del msg['js_property_callbacks']
         return msg
 
-    def _process_param_change(self, msg):
-        msg = super()._process_param_change(msg)
-        if 'value' in msg:
-            value = msg['value']
-            fmt = self._default_mime
-            if isinstance(value, np.ndarray):
-                fmt = 'wav'
-                buffer = self._from_numpy(value)
-                data = b64encode(buffer.getvalue())
-            elif os.path.isfile(value):
-                fmt = value.split('.')[-1]
-                with open(value, 'rb') as f:
-                    data = f.read()
-                data = b64encode(data)
-            elif value.lower().startswith('http'):
-                return msg
-            elif not value or value == f'data:{self._media_type}/{fmt};base64,':
-                data = b''
-            else:
-                raise ValueError(f'Object should be either path to a {self._media_type} file or numpy array.')
-            msg['value'] = f"data:{self._media_type}/{fmt};base64,{data.decode('utf-8')}"
-        return msg
+    def _transform_object(self, obj: Any) -> Dict[str, Any]:
+        fmt = self._default_mime
+        if obj is None:
+            data = b''
+        elif isinstance(obj, (np.ndarray, TensorLike)):
+            fmt = 'wav'
+            buffer = self._to_buffer(obj)
+            data = b64encode(buffer.getvalue())
+        elif os.path.isfile(obj):
+            fmt = obj.split('.')[-1]
+            with open(obj, 'rb') as f:
+                data = f.read()
+            data = b64encode(data)
+        elif obj.lower().startswith('http'):
+            return dict(object=obj)
+        elif not obj or obj == f'data:{self._media_type}/{fmt};base64,':
+            data = b''
+        else:
+            raise ValueError(f'Object should be either path to a {self._media_type} file or numpy array.')
+        b64 = f"data:{self._media_type}/{fmt};base64,{data.decode('utf-8')}"
+        return dict(object=b64)
 
+_VALID_TORCH_DTYPES_FOR_AUDIO = [
+    "torch.short", "torch.int16",
+    "torch.half", "torch.float16",
+    "torch.float", "torch.float32",
+    "torch.double", "torch.float64",
+]
+
+_VALID_NUMPY_DTYPES_FOR_AUDIO = [np.int16, np.uint16, np.float32, np.float64]
+
+def _is_1dim_int_or_float_tensor(obj: Any)->bool:
+    return (
+        isinstance(obj, TensorLike) and
+        obj.dim()==1 and
+        str(obj.dtype) in _VALID_TORCH_DTYPES_FOR_AUDIO
+    )
+
+def _is_1dim_int_or_float_ndarray(obj: Any)->bool:
+    return (
+        isinstance(obj, np.ndarray) and
+        obj.ndim==1 and
+        obj.dtype in _VALID_NUMPY_DTYPES_FOR_AUDIO
+    )
 
 class Audio(_MediaBase):
     """
     The `Audio` pane displays an audio player given a local or remote audio
-    file or numpy array.
+    file, a NumPy Array or Torch Tensor.
 
     The pane also allows access and control over the player state including
     toggling of playing/paused and loop state, the current time, and the
     volume.
 
-    The audio player supports ogg, mp3, and wav files as well as numpy arrays.
+    The audio player supports ogg, mp3, and wav files
+
+    If SciPy is installed, 1-dim Numpy Arrays and 1-dim
+    Torch Tensors are also supported. The dtype must be one of the following
+
+    - numpy: np.int16, np.uint16, np.float32, np.float64
+    - torch: torch.short, torch.int16, torch.half, torch.float16, torch.float, torch.float32,
+    torch.double, torch.float64
+
+    The array or Tensor input will be downsampled to 16bit and converted to a wav file by SciPy.
 
     Reference: https://panel.holoviz.org/reference/panes/Audio.html
 
@@ -143,12 +188,12 @@ class Audio(_MediaBase):
     >>> Audio('http://ccrma.stanford.edu/~jos/mp3/pno-cs.mp3', name='Audio')
     """
 
-    object = param.ClassSelector(default='', class_=(str, np.ndarray,),
+    object = param.ClassSelector(default='', class_=(str, np.ndarray, TensorLike),
                                  allow_None=True, doc="""
-        The audio file either local or remote.""")
+        The audio file either local or remote, a 1-dim NumPy ndarray or a 1-dim Torch Tensor.""")
 
     sample_rate = param.Integer(default=44100, doc="""
-        The sample_rate of the audio when given a NumPy array.""")
+        The sample_rate of the audio when given a NumPy array or Torch tensor.""")
 
     _bokeh_model = _BkAudio
 
@@ -160,8 +205,10 @@ class Audio(_MediaBase):
 
     @classmethod
     def applies(cls, obj: Any) -> float | bool | None:
-        return (super().applies(obj) or
-                (isinstance(obj, np.ndarray) and obj.ndim==1 and obj.dtype in [np.int16, np.uint16]))
+        return (super().applies(obj)
+            or _is_1dim_int_or_float_ndarray(obj)
+            or _is_1dim_int_or_float_tensor(obj)
+        )
 
 
 class Video(_MediaBase):
