@@ -2,6 +2,7 @@ import time
 
 import pytest
 
+from bokeh.core.has_props import _default_resolver
 from bokeh.model import Model
 
 pytestmark = pytest.mark.ui
@@ -15,8 +16,7 @@ from panel.pane.ipywidget import Reacton
 def cleanup_ipywidgets():
     old_models = dict(Model.model_class_reverse_map)
     yield
-    Model.model_class_reverse_map = old_models
-
+    _default_resolver._known_models = old_models
 
 def test_reacton(page, port):
     import reacton
