@@ -15,7 +15,9 @@ from bokeh.events import ButtonClick, MenuItemClick
 from bokeh.models import (
     Button as _BkButton, Dropdown as _BkDropdown, Toggle as _BkToggle,
 )
+from bokeh.models.ui import SVGIcon, TablerIcon
 
+from ..io.resources import CDN_DIST
 from ..links import Callback
 from .base import Widget
 
@@ -27,8 +29,8 @@ if TYPE_CHECKING:
     from ..links import JSLinkTarget, Link
 
 
-BUTTON_TYPES: List[str] = ['default', 'primary', 'success', 'warning', 'danger','light']
-
+BUTTON_TYPES: List[str] = ['default', 'primary', 'success', 'warning', 'danger', 'light']
+BUTTON_STYLES: List[str] = ['solid', 'outline']
 
 class _ButtonBase(Widget):
 
@@ -37,17 +39,62 @@ class _ButtonBase(Widget):
         (blue), 'success' (green), 'info' (yellow), 'light' (light),
         or 'danger' (red).""")
 
-    _rename: ClassVar[Mapping[str, str | None]] = {'name': 'label'}
+    button_style = param.ObjectSelector(default='solid', objects=BUTTON_STYLES, doc="""
+        A button style to switch between 'solid', 'outline'.""")
+
+    _rename: ClassVar[Mapping[str, str | None]] = {'name': 'label', 'button_style': None}
+
+    _source_transforms: ClassVar[Mapping[str, str | None]] = {'button_style': None}
+
+    _stylesheets: ClassVar[List[str]] = [f'{CDN_DIST}css/button.css']
 
     __abstract = True
 
+    def _process_param_change(self, params):
+        if 'button_style' in params or 'css_classes' in params:
+            params['css_classes'] = [
+                params.pop('button_style', self.button_style)
+            ] + params.get('css_classes', self.css_classes)
+        return super()._process_param_change(params)
 
-class _ClickButton(_ButtonBase):
+
+class IconMixin(Widget):
+
+    icon = param.String(default='', doc="""
+        An icon to render to the left of the button label. Either an SVG or an
+        icon name which is loaded from https://tabler-icons.io/.""")
+
+    icon_size = param.String(default='1em', doc="""
+        Size of the icon as a string, e.g. 12px or 1em.""")
+
+    _rename: ClassVar[Mapping[str, str | None]] = {
+        'icon_size': None
+    }
+
+    __abstract = True
+
+    def _process_param_change(self, params):
+        icon_size = params.pop('icon_size', self.icon_size)
+        if params.get('icon') is not None:
+            icon = params['icon']
+            if icon.lstrip().startswith('<svg'):
+                icon_model = SVGIcon(svg=icon, size=icon_size)
+            else:
+                icon_model = TablerIcon(icon_name=icon, size=icon_size)
+            params['icon'] = icon_model
+        return super()._process_param_change(params)
+
+
+class _ClickButton(_ButtonBase, IconMixin):
 
     __abstract = True
 
     _event: ClassVar[str] = 'button_click'
 
+    _source_transforms: ClassVar[Mapping[str, str | None]] = {
+        'button_style': None, 'icon_size': None, 'icon': None
+    }
+
     def _get_model(
         self, doc: Document, root: Optional[Model] = None,
         parent: Optional[Model] = None, comm: Optional[Comm] = None
@@ -129,7 +176,9 @@ class Button(_ClickButton):
     value = param.Event(doc="""
         Toggles from False to True while the event is being processed.""")
 
-    _rename: ClassVar[Mapping[str, str | None]] = {'clicks': None, 'name': 'label', 'value': None}
+    _rename: ClassVar[Mapping[str, str | None]] = {
+        'clicks': None, 'name': 'label', 'value': None
+    }
 
     _target_transforms: ClassVar[Mapping[str, str | None]] = {
         'event:button_click': None, 'value': None
@@ -204,7 +253,7 @@ class Button(_ClickButton):
         return self.param.watch(callback, 'clicks', onlychanged=False)
 
 
-class Toggle(_ButtonBase):
+class Toggle(_ButtonBase, IconMixin):
     """The `Toggle` widget allows toggling a single condition between `True`/`False` states.
 
     This widget is interchangeable with the `Checkbox` widget.
@@ -219,7 +268,9 @@ class Toggle(_ButtonBase):
     value = param.Boolean(default=False, doc="""
         Whether the button is currently toggled.""")
 
-    _rename: ClassVar[Mapping[str, str | None]] = {'value': 'active', 'name': 'label'}
+    _rename: ClassVar[Mapping[str, str | None]] = {
+        'value': 'active', 'name': 'label', 'icon': None, 'icon_size': None
+    }
 
     _supports_embed: ClassVar[bool] = True
 
